<?php

require_once(dirname(__FILE__).'/PermissionsException.php');

class FilesHelper
{
    public static function removeDir($dir, $self = true, $excluded = array())
    {
        try
        {
           $it = new RecursiveDirectoryIterator($dir, RecursiveDirectoryIterator::SKIP_DOTS);
            $files = new RecursiveIteratorIterator($it,
                         RecursiveIteratorIterator::CHILD_FIRST);
            foreach($files as $file) {
                if (count($excluded) > 0) {
                    $parts = preg_split('/\\\\|\//', $file->getRealPath());
                    if (count(array_intersect($excluded, $parts)) > 0) {
                        continue;
                    }
                }
                if ($file->isDir()) {
                    FilesHelper::deleteFolder($file->getRealPath());
                } else {
                    FilesHelper::deleteFile($file->getRealPath());
                }
            }
            if ($self) FilesHelper::deleteFolder($dir);
        }
        catch (Exception $e)
        {
            throw new PermissionsException('Unable to remove the directory: ' . $dir);
        }
    }

    public static function enumerateRecursive($path)
    {
        $files = array();
        if (!file_exists($path) || self::isEmptyDir($path))
            return $files;
        $flags = FilesystemIterator::CURRENT_AS_FILEINFO | FilesystemIterator::SKIP_DOTS | FilesystemIterator::UNIX_PATHS;
        $iterator = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($path, $flags));
        foreach ($iterator as $fileInfo) {
            $files[$iterator->getSubPathname()] = FilesHelper::readFile($fileInfo->getPathName());
        }
        return $files;
    }

    /**
     * @param $path
     * @param $recursive
     */
    public static function enumerateFiles($path, $recursive = true) {
        $files = array();
        if (!file_exists($path) || self::isEmptyDir($path))
            return $files;
        $flags = FilesystemIterator::CURRENT_AS_FILEINFO | FilesystemIterator::SKIP_DOTS | FilesystemIterator::UNIX_PATHS;
        $iterator = new IteratorIterator(new FilesystemIterator($path, $flags));
        foreach ($iterator as $fileInfo) {
            if ($fileInfo->isFile()) {
                $path = $fileInfo->getPathName();
                $files[] = array('path' => $path, 'name' => $fileInfo->getFilename(), 'content' => FilesHelper::readFile($path));
            }
        }
        return $files;
    }

        /**
     * @param $path
     * @param $recursive
     */
    public static function checkFiles($path, $recursive = true, $check = false, $rootOnly = false, $names_only = false) {
        $files = array();
        if (!is_dir($path)) {
            return $files;
        }

        if ($check && ((is_dir($path) && !self::_checkDirPermissions($path)) || !self::_checkFilePermissions($path)))
            throw new PermissionsException('Please check permissions for: ' . $path);

        if ($rootOnly)
            return $files;

        if ($handle = opendir($path)) {
            while (($name = readdir($handle)) !== false) {
                if (preg_match('#^\.#', $name)) {
                    continue;
                }

                if (is_dir($path . "/" . $name) && $recursive) {
                    if ($check && !self::_checkDirPermissions($path))
                        throw new PermissionsException('Please check permissions for: ' . $path);

                    $files = array_merge($files, FilesHelper::checkFiles($path . "/" . $name, $recursive, $check, $rootOnly, $names_only));
                } else {
                    if ($check && !self::_checkFilePermissions($path))
                        throw new PermissionsException('Please check permissions for: ' . $path);

                    if ($names_only)
                        $files[] = $name;
                    else
                        $files[] = array('path' => $path . '/' . $name, 'name' => $name);
                }
            }
            closedir($handle);
        }

        return $files;
    }

    /**
     * @param $source
     * @param $destination
     */
    public static function copyRecursive($source, $destination, $change_file = null) {
        if(is_file($source)) {
            $ext = pathinfo($source, PATHINFO_EXTENSION);
            if ($change_file && in_array($ext, array('tpl'))) {
                self::writeFile($destination, call_user_func($change_file, $source));
            } else {
                self::copyFile($source, $destination);
            }
        } elseif(is_dir($source)) {
            self::createDir($destination);

            if ($dh = opendir($source)) {
                while (($file = readdir($dh)) !== false) {
                    if('.' == $file || '..' == $file) {
                        continue;
                    }
                    FilesHelper::copyRecursive($source . '/' . $file, $destination . '/' . $file, $change_file);
                }
                closedir($dh);
            }
        }
    }

    public static function normalizePath($path) {
        $root = ($path[0] === '/') ? '/' : '';
        $segments = preg_split('/[\\/\\\\]/', trim($path, '/'));
        $ret = array();
        foreach ($segments as $segment) {
            if (($segment == '.') || empty($segment)) {
                continue;
            }
            if ($segment == '..') {
                array_pop($ret);
            } else {
                array_push($ret, $segment);
            }
        }
        return $root . implode('/', $ret);
    }

    public static function checkDirPermissions($path)
    {
        return is_dir($path) && is_writable($path) && is_readable($path);
    }

    public static function checkFilePermissions($path)
    {
        return file_exists($path) && is_writable($path) && is_readable($path);
    }

    public static function createDir($path)
    {
        if (!file_exists($path)) {
            @mkdir($path, 0777, true);
            if (!is_dir($path) && !file_exists($path)) {
                throw new PermissionsException('Unable to create the directory: ' . $path);
            }
        }
    }

    public static function writeFile($path, $content, $flags = 0)
    {
        set_error_handler('permissionErrorHandler');
        $message = 'Unable to create or write to the file: ';
        $dir = dirname($path);
        self::createDir($dir);
        $chmodResult = true;
        if (file_exists($path)) {
            $chmodResult = chmod($path, 0755);
        }
        $return = file_put_contents($path, $content, $flags);
        if (false === $return || !$chmodResult) {
            throw new PermissionsException($message . $path);
        }
        restore_error_handler();
        chmod($path, 0755);
    }

    public static function readFile($path)
    {
        set_error_handler('permissionErrorHandler');
        $message = 'Unable to read the file: ';
        $e = new PermissionsException($message . $path);
        if (!file_exists($path) || !is_readable($path))
            throw $e;
        $content = file_get_contents($path);
        if (false === $content)
            throw $e;

        restore_error_handler();
        return $content;
    }

    public static function renameFile($oldname, $newname)
    {
        if (!rename($oldname, $newname)) {
            throw new PermissionsException('Unable to rename the file: ' . $oldname);
        }
    }

    public static function copyFile($source, $dest)
    {
        $dir = dirname($dest);
        self::createDir($dir);
        if (!copy($source, $dest)) {
            throw new PermissionsException('Unable to copy the file: ' . $source);
        }
    }

    public static function deleteFolder($path) {
        @rmdir($path);
        if (is_dir($path) && file_exists($path)) {
            throw new PermissionsException('Unable to remove the folder: ' . $path);
        }
    }

    public static function deleteFile($file)
    {
        if (is_file($file))
            @unlink($file);
        if (file_exists($file)) {
            throw new PermissionsException('Unable to remove the file: ' . $file);
        }
    }

    public static function isEmptyDir($dir) {
        if (!is_readable($dir)) return NULL;
            $handle = opendir($dir);
        while (false !== ($entry = readdir($handle))) {
            if ($entry != "." && $entry != "..") {
                return FALSE;
            }
        }
        return TRUE;
    }

    public static function emptyDirRecursive($dir, $recursive = true) {
        foreach (FilesHelper::enumerateFiles($dir, $recursive) as $file)
            FilesHelper::deleteFile($file['path']);
    }

    public static function _checkDirPermissions($dir)
    {
        return is_dir($dir) && is_writable($dir) && is_readable($dir);
    }

    public static function _checkFilePermissions($path)
    {
        return file_exists($path) && is_writable($path) && is_readable($path);
    }
}

function permissionErrorHandler($errno, $errstr, $errfile, $errline, $errcontext)
{
    throw new PermissionsException($errcontext['message'] . $errcontext['path']);
}

function fatalErrorShutdownHandler()
{
    $last_error = error_get_last();
    if ($last_error['type'] === E_ERROR) {
        // fatal error
        errorHandler(E_ERROR, 'Fatal error: ' . $last_error['message'], $last_error['file'], $last_error['line'], array());
        exit();
    }
}

function errorHandler($errno, $errstr, $errfile, $errline, $errcontext)
{
    if (!(error_reporting() & $errno)) {
        // This error code is not included in error_reporting
        return;
    }
    ob_start();
    switch ($errno) {
        case E_USER_ERROR:
            ob_start();
            echo "USER ERROR: $errstr\n";
            echo "  Fatal error on line $errline in file $errfile";
            echo ", PHP " . PHP_VERSION . " (" . PHP_OS . ")\n";
            echo "Aborting...\n";
            break;
        case E_USER_WARNING:
            echo "USER WARNING: $errstr<br />\n";
            break;
        case E_USER_NOTICE:
            echo "USER NOTICE: $errstr<br />\n";
            break;
        default:
            $errorType = friendlyErrorType($errno);
            echo ("" !== $errorType ? "$errorType: " : "Unknown error type: ") . "$errstr $errfile $errline\n";
            break;
    }
    $error = ob_get_clean();

    $bt = debug_backtrace();
    $formatter = new LogFormatter();
    $callstack = '';
    foreach($bt as $key => $caller) {
        if (0 === $key) continue;
        $callstack .= "   -> ";
        if (isset($caller['class']))
            $callstack .= $caller['class'] . '::';
        $callstack .= $caller['function'] . '(' . $formatter->args($caller['args']) . ')';
        $callstack .= "\n";
    }
    $os = 'OS: ' . php_uname() . "\n";
    $error = "\n\n  " . $os . "\n" . $error . "\n" . print_r($errcontext, true) . "  Callstack: \n" . $callstack . "\n\n";
    header("HTTP/1.1 400 Request failed");
    exit($error);

	/* Don't execute PHP internal error handler */
    return true;
}

function friendlyErrorType($type)
{
    switch($type)
    {
        case E_ERROR: // 1 //
            return 'E_ERROR';
        case E_WARNING: // 2 //
            return 'E_WARNING';
        case E_PARSE: // 4 //
            return 'E_PARSE';
        case E_NOTICE: // 8 //
            return 'E_NOTICE';
        case E_CORE_ERROR: // 16 //
            return 'E_CORE_ERROR';
        case E_CORE_WARNING: // 32 //
            return 'E_CORE_WARNING';
        case E_CORE_ERROR: // 64 //
            return 'E_COMPILE_ERROR';
        case E_CORE_WARNING: // 128 //
            return 'E_COMPILE_WARNING';
        case E_USER_ERROR: // 256 //
            return 'E_USER_ERROR';
        case E_USER_WARNING: // 512 //
            return 'E_USER_WARNING';
        case E_USER_NOTICE: // 1024 //
            return 'E_USER_NOTICE';
        case E_STRICT: // 2048 //
            return 'E_STRICT';
        case E_RECOVERABLE_ERROR: // 4096 //
            return 'E_RECOVERABLE_ERROR';
        case E_DEPRECATED: // 8192 //
            return 'E_DEPRECATED';
        case E_USER_DEPRECATED: // 16384 //
            return 'E_USER_DEPRECATED';
    }
    return "";
}